#!/usr/bin/python
#-*- coding: utf-8 -*-

# Soubor .osm (OpenStreetMaps) obsahuje nasledujiich 6 zakladnich tagu:
#   node - popisuje bod na mape
#   way - seznam bodu
#     nd - jeden bod cesty (way)
#   relation - shlukuje jine objekty (relation, way nebo node)
#     member - popisuje prvky relation
#   tag - pripisuje vlastnosti typu node, way nebo relation (napr. jmeno)

# Prvni faze extrahovani geografickych udaju.
# Vytvori ze zadaneho xml soubor ke zpracovani, ve kterem jsou jednotlive
# "relation" a "way" zapsany seznamem clenu (jejich id).
# V dalsi fazi se tento soubor bude zpracovavat a nahrazovat posloupnosti
# primo geografickymi souradnicemi.

# vygenerovany temp_file obsahuje radky dvou typu:
#   id 0 lat lon
#   id n 1.clen 2.clen ... n.clen

import sys

def usage():
  sys.stderr.write("Pouziti: %s osm_filename temp_filename" % sys.argv[0])
  sys.exit()

if len(sys.argv) != 3: usage()
FILE_NAME = sys.argv[1]
temp_file = open(sys.argv[2], "w")

import xml.sax
class Handler (xml.sax.handler.ContentHandler):
  # Nasledujici promenne se vztahuji k aktualnimu objektu
  current_id = 0
  lat, lon = 0, 0
  members = []

  nodes_proceeded = 0

  def startElement (self, name, attrs):
    if name in ("node", "way", "relation"):
      self.current_id = int(attrs.get('id'))
      self.members = []
    if name == "node":
      self.lat, self.lon = float(attrs.get('lat')), float(attrs.get('lon'))
    elif name in ("member", "nd"):
      self.members.append(int(attrs.get('ref')))

  def endElement (self, name, attrs = None):
    if name in ("node",):
      temp_file.write("%d 0 %.7lf %.7lf\n" % (self.current_id, self.lat, self.lon))
    elif name in ("way", "relation"):
      temp_file.write("%d %d" % (self.current_id, len(self.members)))
      for member in self.members: temp_file.write(" %s" % member)
      temp_file.write("\n")

    if name in ("node", "way", "relation"):
      self.nodes_proceeded += 1
      if self.nodes_proceeded % 10 ** 6 == 0:
        sys.stderr.write("%d nodes proceeded\n" % self.nodes_proceeded)

xml.sax.parse (FILE_NAME, Handler())

temp_file.close()
