#!/bin/sh

# Tento skript slouzi k testovani ruznych konfiguraci desifratoru.
# Slouzi vyhradne pro vyvojove ucely a ocekava se, ze bude spusten
# z korenoveho adresare projektu.

# Jeho jedinym argumentem je cesta k aktualnimu konfiguracnimu souboru

CONFIG_TEMPLATE=$1
CONFIG_FILENAME=config.h

EXECUTABLE=./decipher
SCORER=script/score.py

TRAIN_INPUT=training_coded
TRAIN_GEOGRAPHY=solutions/training_geography
TRAIN_OUTPUT=solutions/training_divided

DEV_INPUT=development_coded
DEV_GEOGRAPHY=solutions/development_geography
DEV_OUTPUT=solutions/development_divided

EVAL_INPUT=solutions/evaluation/evaluation_coded
EVAL_GEOGRAPHY=solutions/evaluation/evaluation_geography
EVAL_OUTPUT=solutions/evaluation/evaluation

RESULTS_DIR=results/

# Vytvori konfiguracni soubor CONFIG_FILENAME podle vzoru CONFIG_TEMPLATE.
# Nahradi v nem navic radek cislo $1 radkem $2.
create_config_file () {
  head -n $(( $1 - 1)) $CONFIG_TEMPLATE > $CONFIG_FILENAME
  echo $2 >> $CONFIG_FILENAME
  tail -n +$(( $1 + 1)) $CONFIG_TEMPLATE >> $CONFIG_FILENAME
}

# Spusti test na cele sade dat. Ocekava 5 parametru:
#   $1 - vstupni sada testovacich dat
#   $2 - odpovidajici spravna raseni sifer
#   $3 - nazev vystupniho souboru v RESULTS_DIR
#   $4 - cislo radku v konfiguracnim souboru
#   $5 - cim se ma nahradit radek cislo $4 v konfiguracnim souboru
run_test() {
  echo ""
  echo "--------------- Running new test ---------------"
  echo "- Test name: $3"
  echo ""

  create_config_file $4 "$5"
  make
  echo ""
  $SCORER $EXECUTABLE $1 $2 | tee $RESULTS_DIR$3
}

run_test $DEV_INPUT $DEV_OUTPUT dev_N_MAX_1 11 "const unsigned N_MAX = 1;"
run_test $DEV_INPUT $DEV_OUTPUT dev_N_MAX_3 11 "const unsigned N_MAX = 3;"
run_test $DEV_INPUT $DEV_OUTPUT dev_N_MAX_INF 11 "const unsigned N_MAX = 100;"

run_test $TRAIN_INPUT $TRAIN_OUTPUT train_N_MAX_1 11 "const unsigned N_MAX = 1;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_N_MAX_3 11 "const unsigned N_MAX = 3;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_N_MAX_INF 11 "const unsigned N_MAX = 100;"

run_test $DEV_INPUT $DEV_OUTPUT dev_COST_DIFF_50 33 "const unsigned COST_DIFF = 50;"
run_test $DEV_INPUT $DEV_OUTPUT dev_COST_DIFF_100 33 "const unsigned COST_DIFF = 100;"
run_test $DEV_INPUT $DEV_OUTPUT dev_COST_DIFF_150 33 "const unsigned COST_DIFF = 150;"
run_test $DEV_INPUT $DEV_OUTPUT dev_COST_DIFF_200 33 "const unsigned COST_DIFF = 200;"
run_test $DEV_INPUT $DEV_OUTPUT dev_COST_DIFF_250 33 "const unsigned COST_DIFF = 250;"
run_test $DEV_INPUT $DEV_OUTPUT dev_COST_DIFF_300 33 "const unsigned COST_DIFF = 300;"

run_test $TRAIN_INPUT $TRAIN_OUTPUT train_COST_DIFF_50 33 "const unsigned COST_DIFF = 50;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_COST_DIFF_100 33 "const unsigned COST_DIFF = 100;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_COST_DIFF_150 33 "const unsigned COST_DIFF = 150;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_COST_DIFF_200 33 "const unsigned COST_DIFF = 200;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_COST_DIFF_250 33 "const unsigned COST_DIFF = 250;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_COST_DIFF_300 33 "const unsigned COST_DIFF = 300;"

run_test $DEV_INPUT $DEV_OUTPUT dev_PHASE_1 14 "const bool PHASE_2 = false;"
run_test $DEV_INPUT $DEV_OUTPUT dev_PHASE_2 13 "const bool PHASE_1 = false;"

run_test $TRAIN_INPUT $TRAIN_OUTPUT train_PHASE_1 14 "const bool PHASE_2 = false;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_PHASE_2 13 "const bool PHASE_1 = false;"

run_test $DEV_INPUT $DEV_OUTPUT dev_ALG_BFS 18 "const unsigned ALG = ALG_BFS;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_ALG_BFS 18 "const unsigned ALG = ALG_BFS;"

run_test $DEV_INPUT $DEV_OUTPUT dev_MAX_LEN_GEN_VARIANTS_7 21 "const unsigned MAX_LENGTH_TO_GENERATE_VARIANTS = 7;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_MAX_LEN_GEN_VARIANTS_7 21 "const unsigned MAX_LENGTH_TO_GENERATE_VARIANTS = 7;"

run_test $DEV_INPUT $DEV_OUTPUT dev_ORDER_C 28 "const unsigned ORDER = ORDER_C;"
run_test $DEV_INPUT $DEV_OUTPUT dev_ORDER_L 28 "const unsigned ORDER = ORDER_L;"

run_test $TRAIN_INPUT $TRAIN_OUTPUT train_ORDER_C 28 "const unsigned ORDER = ORDER_C;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_ORDER_L 28 "const unsigned ORDER = ORDER_L;"

run_test $DEV_INPUT $DEV_OUTPUT dev_MAX_TIME_300 31 "const unsigned MAX_TIME1 = 60, MAX_TIME2 = 300;"
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_MAX_TIME_300 31 "const unsigned MAX_TIME1 = 60, MAX_TIME2 = 300;"

run_test $DEV_INPUT $DEV_OUTPUT dev_INITIAL 3 " "
run_test $TRAIN_INPUT $TRAIN_OUTPUT train_INITIAL 3 " "

run_test $DEV_GEOGRAPHY $DEV_OUTPUT dev_GEOGRAPHY_Y 3 " "
run_test $TRAIN_GEOGRAPHY $TRAIN_OUTPUT train_GEOGRAPHY_Y 3 " "

run_test solutions/development_geography_n $DEV_OUTPUT dev_GEOGRAPHY_N 3 " "
run_test solutions/training_geography_n $TRAIN_OUTPUT train_GEOGRAPHY_N 3 " "

# Zaverecne vyhodnoceni na evaluacnich datech

# run_test $EVAL_GEOGRAPHY $EVAL_OUTPUT eval_GEOGRAPHY_Y 3 " "
# run_test solutions/evaluation/evaluation_geography_n $EVAL_OUTPUT eval_GEOGRAPHY_N 3 " "

# run_test $EVAL_INPUT $EVAL_OUTPUT eval_FINAL 3 " "


