#!/usr/bin/python
#-*- coding: utf-8 -*-

# V zadanem textu spocita kolik se nachazi jednotlivych N-gramu podle danych kategorii
# http://en.wikipedia.org/wiki/N-gram

# Pocita N-gramy pro N v nasledujicim rozsahu
MIN_N, MAX_N = 1, 3

import sys, os, string, math
SCRIPT_DIR = os.path.dirname(sys.argv[0])
WORDLIST_DIR = os.path.join(SCRIPT_DIR, '..', 'wordlists')
CONFIG_FILENAME = os.path.join(WORDLIST_DIR, 'categories.cfg')

EXCLUDED_WORDLISTS = [] # ("ostatni")

recognized_lines, nonrecognized_lines = 0, 0

# Skript bud ukazuje slova, ktera nepatri do zadne kategorie nebo pocita n-gramy
SHOW_UNKNOWN = True

def usage():
  print("Pouziti: %s filename" % sys.argv[0])
  sys.exit()

if len(sys.argv) != 2: usage()

FILENAME_TO_ANALYZE = sys.argv[1]

# Nacteni slovnich trid
wordlists = {}
with open(CONFIG_FILENAME, "r") as config:
  for wordlist_filename in map(string.strip, config.readlines()):
    if wordlist_filename in EXCLUDED_WORDLISTS: continue
    with open(os.path.join(WORDLIST_DIR, wordlist_filename), "r") as wordlist_file:
      wordlists[wordlist_filename] = set(map(string.strip, wordlist_file.readlines()))

count = {}
for n in range(MIN_N, MAX_N + 1): count[n] = {}

# Rekurzivni funkce, ktera pocita n-gramy
def calculate_n_grams(n, start, n_gram_part, categories):
  if len(n_gram_part) == n:
    n_gram = tuple(n_gram_part)
    if count[n].get(n_gram) == None: count[n][n_gram] = 0
    count[n][n_gram] += 1
    return

  position = len(n_gram_part) + start
  if (position >= len(categories)): return
  for category in categories[position]:
    calculate_n_grams(n, start, n_gram_part + [category], categories)

with open(FILENAME_TO_ANALYZE, "r") as file_to_analyze:
  for line in map(string.strip, file_to_analyze.readlines()):
    words = line.split()
    categories = []
    for word in words:
      categories.append([])
      for wordlist in wordlists.keys():
        if word in wordlists[wordlist]:
          categories[-1].append(wordlist)

    if SHOW_UNKNOWN:
      OKGREEN = '\033[92m'
      FAIL = '\033[91m'
      ENDC = '\033[0m'
      # if [] in categories:
      for i in range(len(words)):
        if categories[i]: print OKGREEN + words[i] + ENDC,
        else: print FAIL + words[i] + ENDC,
      print ""

      if [] in categories: nonrecognized_lines += 1
      else: recognized_lines += 1

    for n in range(MIN_N, MAX_N + 1):
      for start in range(len(words)):
        calculate_n_grams(n, start, [], categories)

if SHOW_UNKNOWN:
  print ""
  print "Rozpoznanych radek: %d" % recognized_lines
  print "Nerozpoznanych radek: %d" % nonrecognized_lines
  print ""

else:
  for n in range(MIN_N, MAX_N + 1):
    n_grams = [(cnt, n_gram) for n_gram, cnt in count[n].items()]
    n_grams.sort()
    print "n = %d" % n
    for n_gram in n_grams:
      print "%3d   %s" % (n_gram[0], " + ".join(n_gram[1]))
