#!/usr/bin/python
#-*- coding: utf-8 -*-

# Tento skript stahuje ze stranek sifrovacich her reseni sifer a vypisuje je na stdout.
# Navic vraci vysledky v jednotnem formatu a melo by byt snadne pridat dalsi sifrovaci hry.

import re, HTMLParser
from functions import *

HAND_COLLECTED_FILENAME = "../solutions/hand_collected/"

# Funkce k automatickemu stahavani reseni sifer ze stranek sifrovacich her
# Parametry:
#   ciphers_list_URL - url stranky, na ktere se seznam sifer
#   ciphers_pattern_regex - regularni vyraz, kterym se na strance urci odkazy na reseni jednotlivych sifer
#   solutions_pattern_regex - regularni vyraz, kterym se ze stranky s resenim vyextrahuje samotne reseni
#   ciphers_URL - odkaz na stranku s resenim ziskany pomoci ciphers_pattern_regex casto neni kompletni,
#                 a proto je mozne ho doplnit timto parametrem (pokud je tam jenom relativni adresa)
def get_solutions(ciphers_list_URL, ciphers_pattern_regex, solutions_pattern_regex, ciphers_URL = '%s'):
  page = download_page(ciphers_list_URL)

  ciphers_pattern = re.compile(ciphers_pattern_regex)
  solution_pattern = re.compile(solutions_pattern_regex)

  ciphers = ciphers_pattern.findall(page)
  # v poli ciphers jsou odkazy na jednotlive stranky s resenimi
  for c in ciphers:
    url = ciphers_URL % HTMLParser.HTMLParser().unescape(c)
    print url,
    if (len(c) >= 4 and c[-4:].lower() in ('.pdf', '.jpg', '.doc', '.png', '.zip')):
      cipher = ""
    else:
      try: cipher = download_page(url).replace('\n', ' ')
      except HTTPError:
        print 'Not Found.'
        continue
    solution = solution_pattern.findall(cipher)
    if(len(solution)):
      sol = solution[0]
      if(type(sol) == tuple): sol = sol[-1]
      sol = normalize(sol)
      if(sol): print sol
      else: print "" # prazdny text (nejspis obrazek) -> je potreba to najit rucne
    else: print "" # regex vubec nenalezen na strance

# Bere reseni sifer ze zadaneho souboru ve slozce data/solution/
def get_from_file(filename, URL):
  f = open(HAND_COLLECTED_FILENAME + filename, 'r')
  solutions = f.readlines()
  for solution in solutions:
    solution = solution.strip()
    if(solution != ''): print URL, solution

# Po trati (2012 - 2013)
# Bedna (2002 - 2012)
# Seslost (2009 - 2011)
# Kruta (za)kruta (1-6. rocnik)
# Palapeli (2013)
# Po skole (2009 - 2012)
# Rozjezd (2012)
# iNula (1 - 6)
# Studna (2003, 2005 - 2011)
# Plzenska sifrovacka
# Paralaxa (2011 - 2012)
# Lost in Hvozd
# NaPALMne (2012)
# Budejovicke zelvovani (2012)
# Leonardo (1 - 3)
# Matrix (2005 - 2012)
# Hradecka sova (2005 - 2012)
# Bota (2010 - 2012)
# Noc tapuru (2010 - 2012)
# TMOU (2004 - 2012)

### Po trati (2012 - 2013) ###
for year in range(2012, 2013 + 1):
  get_solutions(
                'http://www.potrati.cz/%d/sifry' % year,
                '"(/%d/sifry/.*?)"' % year,
                '<div id="solution" style="display: none"><p>(.*?)</p></div>',
                'http://www.potrati.cz%s'
               )

### Bedna (2002 - 2012) ###
for year in range(2002, 2012 + 1):
  get_solutions(
                'http://www.bedna.org/%d/sifry' % year,
                '<a href="(http://www.bedna.org/%d/sifry/[^"]*?)">' % year,
                '<div class="code-solution" style="display: none">(.*?)</div>',
               )

### Seslost (2009 - 2011) ###
for year in range(2009, 2011 + 1):
  get_solutions(
                'http://seslost.cz/%d/sifry' % year,
                '<a href="([^"]*/sifry/[^"]*)">',
                '<div id="solution"[^>]*><p>(.*?)</p></div>',
                'http://seslost.cz%s'
               )

### Kruta (za)kruta (1-6. rocnik) ###
for year in range(1, 6 + 1):
  get_solutions(
                'http://krutazakruta.sifrovacky.cz/?rocnik=%d&stranka=sifry' % year,
                u'<a href="(.*)">řešení</a>',
                'Tajenka:</b>(.*?)<'
               )

### Palapeli (2013) ###
get_solutions(
              'http://palapeli.cz/2/sifry.php',
              "<a href='([^']*)#s_nadpis'>",
              u'(Řešením? je |Řešení: )(.*?)</strong>',
              'http://palapeli.cz/2/%s&reseni=1#s_reseni'
             )

### Po skole (2009 - 2012)
for year in range(2009, 2012 + 1):
  get_solutions(
                'http://poskole.podrate.cz/%d/sifry.cgi' % year,
                u'<a href="?([^">]*)"?>Řešení</a>',
                u'(Řešením? je |Řešení: )(.*?)\\.',
                'http://poskole.podrate.cz/'+str(year)+'/%s'
               )

### Rozjezd (2012) ###
get_solutions(
              'http://www.rozjezd.cz/controls.php?c=1&mode=r',
              "<a href='(controls.php\\?c=.*?&mode=r)'>",
              u'(Řešením? je |Řešení: )(.*?)\\.',
              'http://www.rozjezd.cz/%s'
             )

### iNula (1 - 6) ###
for year in range(1, 6 + 1):
  get_solutions(
                'http://inula.sifrovacky.cz/sifry.php?rocnik=%d' % year,
                '"(sifry.php\\?sifra=sifra[^"]*)"',
                u'(Řešením? je |Řešení: )(.*?)\\.',
                'http://inula.sifrovacky.cz/%s'
               )

### Studna (2003, 2005 - 2011) ###
for year in range(8):
  get_solutions(
                'http://www.studna.zizkov.org/pages/studna%02d.htm' % year,
                u'<a href="([^"]*)">Řešení</a>',
                'obrazek',
                'http://www.studna.zizkov.org/pages/%s'
               )

### Plzenska sifrovacka ###
get_solutions(
              'http://plzenska-sifrovacka.webnode.cz/reseni-sifer/sifry/',
              '<a href="([^"]*_komplet.pdf)">',
              'PDF'
             )

### Paralaxa (2011 - 2012) ###
for year in range(2011, 2012 + 1):
  get_solutions(
                'http://paralaxa.chim.cz/%d/?page=sifry' % year,
                u'<a href="([^"]*)" target="_blank">řešení</a>',
                'PDF'
               )

### Lost in Hvozd ###
get_solutions(
              'http://lostinhvozd.wz.cz/7DC/sifry.php',
              '<a href="(sifry/reseni.*?)"',
              'PDF',
              'http://lostinhvozd.wz.cz/7DC/%s'
             )

### NaPALMne (2012) ###
for year in range(2012, 2012 + 1):
  get_solutions(
                'http://www.napalmne.cz/%d/sifry.php' % year,
                u'<a href="([^"]*)">řešení</a>',
                'PDF',
                'http://www.napalmne.cz/'+str(year)+'/%s'
               )

### Budejovicke zelvovani (2012) ###
for year in range(2012, 2012 + 1):
  get_solutions(
                'http://sifrovackacb.wz.cz/%d/sifry.php' % year,
                u'<td><a href="(.*)" target="_blank">řešení</a></td>',
                'it is PDF',
                'http://sifrovackacb.wz.cz/'+str(year)+'/%s'
               )

### Leonardo (1 - 3) ###
for year in range(3, 5 + 1):
  get_solutions(
                'http://leonardo.ph7.cz/?q=node/%d' % year,
                'href="([^"]*reseni[^"]*)"',
                'DOC'
               )

### Matrix (2005 - 2012) ###

for year in [2012]:
  get_solutions(
                'http://velkyvuz.cz/matrix/%d/ulohy' % year,
                u'<a href="([^"]*)">řešení</a>',
                u'Řešení: ([^<]*)</p>',
                'http://velkyvuz.cz/matrix/'+str(year)+'/%s'
               )
for year in [2010, 2011]:
  get_solutions(
                'http://velkyvuz.cz/matrix/%d/sifry-aktivity' % year,
                u'<a href="([^"]*)">řešení</a>',
                u'(Řešením? je |Řešení: )(.*?)\\.',
                'http://velkyvuz.cz/matrix/'+str(year)+'/%s'
               )
for year in [2008]:
  get_solutions(
                'http://velkyvuz.cz/matrix/%d/sifry.php' % year,
                u'<a href="([^"]*)">řešení</a>',
                u'(Řešením? je |Řešení: )(.*?)\\.',
                'http://velkyvuz.cz/matrix/'+str(year)+'/%s'
               )
for year in [2007]:
  get_solutions(
                'http://velkyvuz.cz/matrix/%d/popis.html' % year,
                u'<td class="s"[^>]*>[^<]*<a href="([^"]*).html">',
                u'(Řešením? je |Řešení: )(.*?)\\.',
                'http://velkyvuz.cz/matrix/'+str(year)+'/%s_res.html'
               )
for year in [2006]:
  get_solutions(
                'http://velkyvuz.cz/matrix/%d/popis.html' % year,
                '<a href="(sifry/.*?).php">',
                u'(Řešením? je |Řešení: )(.*?)\\.',
                'http://velkyvuz.cz/matrix/'+str(year)+'/%sr.php'
               )
for year in [2005]:
  get_solutions(
                'http://velkyvuz.cz/matrix/%d/sifry.html' % year,
                u'<a href="([^"]*)">řešení</a>',
                u'(Řešením? je |Řešení: )(.*?)\\.',
                'http://velkyvuz.cz/matrix/'+str(year)+'/%s'
               )

### Hradecka sova (2005 - 2012) ###
get_solutions(
              'http://sova.osjak.cz/2005/sifry.html',
              '(PDF/Reseni_.*?.pdf)',
              'it is PDF',
              'http://sova.osjak.cz/2005/%s'
             )
get_solutions(
             'http://sova.osjak.cz/sova/2006/reseni14.html',
             '<a href="(reseni.*?)">',
             u'Řešení:</b>(.*?)</p>',
             'http://sova.osjak.cz/sova/2006/%s'
            )
get_solutions(
             'http://sova.osjak.cz/2007/reseni1.php',
             '<a href="(reseni.*?)">',
             u'Řešení:</b>(.*?)<',
             'http://sova.osjak.cz/2007/%s'
            )

for year in range(2008, 2009 + 1):
  get_from_file('sova_%d' % year, 'http://sova.osjak.cz/%d/reseni.php' % year)

for year in range(2010, 2012 + 1):
   get_solutions(
                 'http://sova.osjak.cz/%d/reseni-1' % year,
                 '<a href="(/%d/reseni-[^"]*?)">' % year,
                 'Tajenka je <strong>(.*?)</strong>',
                 'http://sova.osjak.cz%s'
                )

### Bota (2010 - 2012) ###
for year in range(2010, 2012 + 1):
  get_from_file('bota_%d' % year, 'http://bota.unas.cz/%d/Autorske_reseni_sifer%d.doc' % (year, year))

### Noc tapuru (2010 - 2012) ###
for year in range(2010, 2012 + 1):
  get_from_file('noc_tapuru_%d' % year, 'http://www.noctapuru.cz/%d/checkpoints' % year)

### TMOU (2004 - 2012) ###
for year in range(2004, 2012 + 1):
  get_from_file('tmou_%d' % year, "http://www.tmou.cz/%d/" % year)

