#!/usr/bin/python
#-*- coding: utf-8 -*-

# V zadanem textu spocita kolik se nachazi jednotlivych N-gramu podle danych kategorii
# http://en.wikipedia.org/wiki/N-gram

# Pocita N-gramy pro N v nasledujicim rozsahu
MIN_N, MAX_N = 1, 100

DEBUG = False

import sys, os, string, math
SCRIPT_DIR = os.path.dirname(sys.argv[0])
WORDLIST_DIR = os.path.join(SCRIPT_DIR, '..', 'wordlists')
CONFIG_FILENAME = os.path.join(WORDLIST_DIR, 'categories.cfg')

# Nejmensi mozny pocet vyskytu, abychom brali v uvahu podminenou pravdepodobnost
OCCURANCES_LIMIT = 15

# Cena je na zaver vynasobena touto konstantou k dosazeni pekensich cisel
COST_CONST = 20

EXCLUDED_WORDLISTS = ["ostatni"]
EXCLUDED_COST = [90]

def usage():
  print("Pouziti: %s filename" % sys.argv[0])
  sys.exit()

if len(sys.argv) != 2: usage()

FILENAME_TO_ANALYZE = sys.argv[1]

wordlists = {}
with open(CONFIG_FILENAME, "r") as config:
  for wordlist_filename in map(string.strip, config.readlines()):
    if wordlist_filename in EXCLUDED_WORDLISTS: continue
    with open(os.path.join(WORDLIST_DIR, wordlist_filename), "r") as wordlist_file:
      wordlists[wordlist_filename] = set(map(string.strip, wordlist_file.readlines()))

count = {}
for n in range(MIN_N, MAX_N + 1): count[n] = {}

def calculate_n_grams(n, start, n_gram_part, categories):
  if len(n_gram_part) == n:
    n_gram = tuple(n_gram_part)
    if count[n].get(n_gram) == None: count[n][n_gram] = 0
    count[n][n_gram] += 1
    return

  position = len(n_gram_part) + start
  if (position >= len(categories)): return
  for category in categories[position]:
    calculate_n_grams(n, start, n_gram_part + [category], categories)

wordlists["^"] = {}
wordlists["$"] = {}

with open(FILENAME_TO_ANALYZE, "r") as file_to_analyze:
  for line in map(string.strip, file_to_analyze.readlines()):
    words = line.split()
    categories = []
    for word in words:
      categories.append([])
      for wordlist in wordlists.keys():
        if word in wordlists[wordlist]:
          categories[-1].append(wordlist)

    categories = [["^"]] + categories + [["$"]]

    for n in range(MIN_N, MAX_N + 1):
      for start in range(len(categories)):
        calculate_n_grams(n, start, [], categories)

for wordlist in range(len(EXCLUDED_WORDLISTS)):
  print 1, EXCLUDED_COST[wordlist], EXCLUDED_WORDLISTS[wordlist]

for n in range(MIN_N, MAX_N + 1):
  for n_gram in count[n].keys():
    for wordlist in wordlists.keys():
      new_n_gram = tuple(list(n_gram[:-1]) + [wordlist])
      if new_n_gram not in count[n].keys():
        count[n][new_n_gram] = 0

  n_grams = [(n_gram[:-1], cnt, n_gram) for n_gram, cnt in count[n].items()]
  n_grams.sort()
  # print "n = %d" % n

  for tmp_gram, cnt, n_gram in n_grams:
    # if n_gram[-1] == "^" or n_gram[0] == "$": continue

    total_occurances = 0
    for tmp_gram2, cnt2, n_gram2 in n_grams:
      if n_gram[:-1] == n_gram2[:-1]: total_occurances += cnt2
    if total_occurances < OCCURANCES_LIMIT: continue

    wordlist_size = len(wordlists[n_gram[-1]])
    prob = float(cnt + 0.1) / total_occurances
    cost = -COST_CONST * math.log(prob)

    if not DEBUG:
      print ("%d %d %s" % (n, cost, " ".join(n_gram)))
    else:
      print ("%10.5lf, occurances = %3d, total_occurances: %5d, prob = %.7lf   %s" %
            (cost, cnt, total_occurances, prob,
             "P( " + n_gram[-1] + " | " + " ".join(n_gram[:-1]) + " )"))
