#!/usr/bin/python
#-*- coding: utf-8 -*-

# Dostane program na reseni problemu a seznam vstupnich dat
# Ohodnoti kvalitu programu na danych datech vhodnou metrikou

import sys, string
from subprocess import Popen, PIPE
from datetime import datetime, timedelta

COLORS = True

def usage():
  print("%s executable_solution input_file correct_file" % sys.argv[0])
  sys.exit(0)

if len(sys.argv) != 4: usage()

EXECUTABLE_SOLUTION = sys.argv[1]

def get_full_output(input_string):
  input_args = input_string.split()
  script = [EXECUTABLE_SOLUTION]
  if len(input_args) == 3:
    script.append(input_args[0])
    script.append(input_args[1])
  # script.append("-")

  # print script
  p = Popen(script, stdin=PIPE, stdout=PIPE, stderr=PIPE)
  return p.communicate(input_args[-1])[0]

def extract_output(raw_output):
  answer = []
  for line in raw_output.split("\n"):
    tmp = line.split("-")
    if len(tmp) < 2: continue
    answer.append(tmp[1].strip())
  return answer

# Binarni vysledek = schoduji vs. neschoduji se
def score_function1(user_output, correct_output):
  t1 = user_output.replace(" ", "")
  t2 = correct_output.replace(" ", "")
  return float(t1 == t2)

# Spocita v kolika znacich se schoduji (bez ohledu na mezery)
def score_function2(user_output, correct_output):
  t1 = user_output.replace(" ", "")
  t2 = correct_output.replace(" ", "")
  correct = sum([t1[i] == t2[i] for i in range(len(t1))])
  return float(correct) / len(t1)

# Spocita v kolika znacich se schoduji, ale kazde pismeno pocita pouze jednou
def score_function3(user_output, correct_output):
  t1 = user_output.replace(" ", "")
  t2 = correct_output.replace(" ", "")
  correct = 0
  used = set()
  for i in range(len(t1)):
    if (t1[i] in used): continue
    used.add(t1[i])
    correct += t1[i] == t2[i]
  return float(correct) / len(used)

score_functions = [score_function1, score_function2, score_function3]
total_score = [0, 0, 0]
total_time = timedelta()

# Vraci nejlepsi z resenich a procento, kterym odpovida spravnemu reseni
def best_output(outputs, correct_output, score_function):
  best_output, best_score = "", -1
  for output in outputs:
    score = score_function(output, correct_output)
    if score > best_score:
      best_score = score
      best_output = output
  return best_output, best_score

# Ocekava score v procentech 0 - 1
def format_score(score):
  score *= 100
  string_score = "%5.1lf%%" % score
  if not COLORS: return string_score

  # http://stackoverflow.com/a/18085357
  if score < 50: r, g = 1, score / 50
  else: r, g = (100 - score) / 50, 1

  # http://stackoverflow.com/a/15712766
  color = 16 + 6 ** 2 * int(5 * r) + 6 * int(5 * g)

  return "\033[1m\033[38;5;%dm%s\033[0m" % (color, string_score)

def score(input_string, correct):
  global total_score, total_time

  start_time = datetime.today()
  raw_output = get_full_output(input_string)
  end_time = datetime.today()

  running_time = end_time - start_time
  output_lines = extract_output(raw_output)

  for i in range(len(score_functions)):
    output, score = best_output(output_lines, correct, score_functions[i])
    print("    %s %s" % (output, format_score(score)))
    total_score[i] += score

  print("    " + str(running_time))
  total_time += running_time

with open(sys.argv[2], "r") as input_file:
  inputs = map(string.strip, input_file.readlines())

with open(sys.argv[3], "r") as correct_file:
  correct_outputs = map(string.strip, correct_file.readlines())

for i in range(len(inputs)):
  print("%3d %s" % (i + 1, correct_outputs[i]))
  sys.stdout.flush()
  score(inputs[i], correct_outputs[i])
  print("")
  sys.stdout.flush()

print("Celkovy cas: %s" % str(total_time))
print("Prumerny cas: %s" % str(total_time / len(inputs)))
for score in total_score: print(format_score(score / len(inputs)))
