void approach_bfs (const string &, Solution, ConversionTable);

/* Rozhodne, zda ma jiz skoncit vetveni.  */
bool stop_branching (int time_elapsed, int depth, int options_tried,
                     int cost, int position, const string & input) {

  if (!startTime) return depth > 0; // V prvni fazi se zkousi jenom vsechna prvni slova
  if (time_elapsed <= (int)MAX_TIME1) return false; // Prvni krok druhe faze
  if (time_elapsed >= (int)MAX_TIME2) return true;

  // Pokud ma castecne reseni jiz ted dvakrat vetsi cenu nez by melo nejlepsi
  // dosud nalezene reseni, uz temer nema smysl dale prohledavat.
  if (cost * (int)input.size() > 2 * Solution::all.top().cost * position) {
    if (depth > 0) return true;
  }

  // Postupne prorezavani
  int time_remaining = MAX_TIME2 - time_elapsed;
  int L = sqrt(MAX_TIME2 - MAX_TIME1) / 2 + 1; // Delka kroku
  int max_depth = 0;
  while (time_remaining >= (L - max_depth) * (L - max_depth)) {
    time_remaining -= (L - max_depth) * (L - max_depth);
    max_depth++; // Hledame cislo kroku
  }
  if (depth <= max_depth) return false;
  // Ve hloubce vyzsi nez cislo kroku vyzkousime pouze nekolik moznosti
  int max_options = time_remaining * time_remaining / ((L - max_depth) * (L - max_depth));
  if (options_tried > max_options) return true;
  return false;
}

/*
   Od pozice position v retezci input zkousi dat vsechna slova ze slovniku.
   Pokud tam nejake slovo pasuje (s prihlednutim k aktualni prekl. tabulce),
   tak se upravi prekladova tabulka a fuknce se zavola rekurzivne.
*/
void approach_dfs (const string & input, size_t position,
                   ConversionTable & dictionary, Solution & solution) {
  // kdyz je reseni mene pravdepodobne nez posledni z tech nejlepsich, tak skoncime
  if (Solution::all.size() >= MAX_PRINT_SOLUTION &&
      Solution::all.top()  <= solution) return;

  // zaloha prevodni tabulky
  ConversionTable _dictionary = dictionary;

  vector<Option> words;
  Wordlist::get_words(input, position, dictionary, solution, words);

  progress (input, solution, words, dictionary);

  int options_tried = 0;

  // vyzkousime vsechna slova ze slovniku
  for (size_t i = 0; i < words.size(); i++) {
    int cost = dictionary.match(input, position, words[i], solution);
    if (cost) { // pokud slovo pasuje
      solution.push_back(words[i].word); solution.cost += cost;
      if (position + words[i].word->text.size() == input.size()) solution.complete();
      else approach_dfs (input, position + words[i].word->text.size(),
                         dictionary, solution);
      solution.pop_back(); solution.cost -= cost;

      ++options_tried;

      if (stop_branching(time(NULL) - startTime,
                         solution.words.size(),
                         options_tried,
                         solution.cost,
                         position,
                         input)) {
        dictionary = _dictionary;
        break;
      }
    }

    // vratime prevodni tabulku do puvodniho stavu (ze zalohy)
    dictionary = _dictionary;
  }
}

void approach_dfs (string & input) {
  Solution solution;
  ConversionTable dictionary;
  approach_dfs (input, 0, dictionary, solution);
}
