#include "common.h"

ConversionTable::ConversionTable () { clear(); }

// Vyprazdni prevodni tabulku
void ConversionTable::clear () {
  for (int i = 0; i < P; i++) {
    conversion_table[i] = 0;
    used_letter[i] = false;
  }
}

ConversionTable & ConversionTable::operator= (const ConversionTable & c) {
  for (int i = 0; i < P; i++) {
    conversion_table[i] = c.conversion_table[i];
    used_letter[i] = c.used_letter[i];
  }
  return *this;
}

/*
  Uspeje, pokud je mozne aktualni prevodni tabulku doplnit tak, aby
  slovo word odpovidalo prelozene casti retezce cipher od pozice start.
  (neboli, pokud slovo rozsiruje aktualni castecne reseni)
*/
int ConversionTable::match (const string & cipher,
                            int start,
                            const Option & option,
                            const Solution & solution) {
  // slovo word je delsi nez zbytek retezce cipher
  if (cipher.size() < start + option.word->text.size()) return 0;

  for (size_t i = 0; i < option.word->text.size(); i++) {
    char a = cipher[i + start], A = option.word->text[i];
    if (get(a)) { // preklad tohoto znaku uz je v prekladove tabulce
      if (get(a) != A) return 0;
    }
    else { // novy preklad znaku
      if (!set(a, A)) return 0;
    }
  }

  return option.cost;
}

char ConversionTable::get (char c) const {
  if (c < 'a' || c > 'z' || conversion_table[c-'a'] == 0) return 0;
  return conversion_table[c - 'a'];
}

bool ConversionTable::set (char a, char A) {
  if (a < 'a' || a > 'z' || A < 'A' || A > 'Z' ||
      conversion_table[a - 'a'] || used_letter[A - 'A']) return false;
  conversion_table[a - 'a'] = A;
  used_letter[A - 'A'] = true;
  return true;
}

/* Vypis prevodni tabulky */
ostream & operator<< (ostream & out, const ConversionTable & c) {
  out << "{ ";
  bool printed = false;
  for (char i = 'a'; i <= 'z'; i++) if (c.conversion_table[i - 'a']) {
    if (printed) out << ", ";
    printed = true;
    out << i << " => " << c.conversion_table[i - 'a'];
  }
  out << " }";
  return out;
}
